<?php

namespace Drupal\entity_crud_alter_status_message\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\entity_crud_alter_status_message\Entity\EntityCrudAlterStatusMessage;
use Drupal\entity_crud_alter_status_message\EntityCrudAlterStatusMessageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Entity CRUD Alter Status Message form.
 */
final class EntityCrudAlterStatusMessageForm extends EntityForm {

  /**
   * Entity type manager.
   *
   * @var Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Messenger.
   *
   * @var Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The entity type bundle info.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * The entity crud alter status message manager.
   *
   * @var \Drupal\entity_crud_alter_status_message\EntityCrudAlterStatusMessageManagerInterface
   */
  protected $entityCrudAlterStatusMessageManager;

  /**
   * The token service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * Constructs an EntityCrudAlterStatusMessageForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entityTypeManager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entityTypeBundleInfo
   *   The entity type bundle info.
   * @param \Drupal\entity_crud_alter_status_message\EntityCrudAlterStatusMessageManagerInterface $entityCrudAlterStatusMessageManager
   *   The entity crud alter status message manager.
   * @param \Drupal\Core\Utility\Token $token
   *   The token service.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    MessengerInterface $messenger,
    EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    EntityCrudAlterStatusMessageManagerInterface $entityCrudAlterStatusMessageManager,
    Token $token,
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->messenger = $messenger;
    $this->entityTypeBundleInfo = $entityTypeBundleInfo;
    $this->entityCrudAlterStatusMessageManager = $entityCrudAlterStatusMessageManager;
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity_crud_alter_status_message.manager'),
      $container->get('token')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {

    $form = parent::form($form, $form_state);

    /**
     * @var \Drupal\entity_crud_alter_status_message\Entity\EntityCrudAlterStatusMessage $entity_crud_alter_status_message
     */
    $entity_crud_alter_status_message = $this->entity;

    $form['#prefix'] = '<div id="entity-crud-alter-status-message-add-form-wrapper">';
    $form['#suffix'] = '</div>';

    $form['entity_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity type'),
      '#options' => $this->entityCrudAlterStatusMessageManager->getValidEntityTypes(),
      '#default_value' => $entity_crud_alter_status_message->get('entity_type'),
      '#ajax' => [
        'callback' => '::entityTypeAjaxUpdate',
        'wrapper' => 'entity-crud-alter-status-message-add-form-wrapper',
      ],
      '#required' => TRUE,
    ];

    // Get the bundle types for the selected entity type.
    $bundle_info = $this->entityTypeBundleInfo->getBundleInfo($entity_crud_alter_status_message->get('entity_type'));

    // Use ajax to update the form when the entity type is changed.
    $form['entity_bundle'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity bundle'),
      '#options' => array_map(function ($bundle) {
        return $bundle['label'];
      }, $bundle_info),
      '#default_value' => $entity_crud_alter_status_message->get('entity_bundle'),
      '#required' => TRUE,
    ];

    $form['crud_action'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity action'),
      '#options' => $this->entityCrudAlterStatusMessageManager->getCrudMapping($entity_crud_alter_status_message->get('entity_type')),
      '#default_value' => $entity_crud_alter_status_message->get('crud_action'),
      '#required' => TRUE,
    ];

    $form['message'] = [
      '#type' => 'text_format',
      '#allowed_formats' => ['basic_html', 'plain_text'],
      '#title' => $this->t('Message'),
      '#default_value' => $entity_crud_alter_status_message->getMessage(),
      '#required' => TRUE,
      '#description' => $this->t('The message to display when the entity is created, updated or deleted. You can use tokens to include entity data in your message.'),
    ];

    if ($entity_crud_alter_status_message->get('entity_type')) {
      $form['token_help'] = [
        '#type' => 'details',
        '#title' => $this->t('Available tokens'),
        '#open' => TRUE,
        '#weight' => 100,
      ];
      $form['token_help']['browser'] = [
        '#theme' => 'token_tree_link',
        '#token_types' => [$entity_crud_alter_status_message->get('entity_type')],
        '#global_types' => TRUE,
        '#click_insert' => TRUE,
      ];
    }

    return $form;
  }

  /**
   * Ajax callback to update the form.
   */
  public function entityTypeAjaxUpdate(array &$form, FormStateInterface $form_state) {
    // Get the bundle types for the selected entity type.
    $entity_type = $form_state->getValue('entity_type');
    $bundle_info = $this->entityTypeBundleInfo->getBundleInfo($entity_type);
    $form['entity_bundle']['#options'] = array_map(function ($bundle) {
      return $bundle['label'];
    }, $bundle_info);
    $form['crud_action']['#options'] = $this->entityCrudAlterStatusMessageManager->getCrudMapping($entity_type);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $entity_crud_alter_status_message = $this->entity;
    $status = $entity_crud_alter_status_message->save();
    if ($status === SAVED_NEW) {
      $this->messenger()->addMessage($this->t('Alter status message created.'));
    }
    else {
      $this->messenger()->addMessage($this->t('Alter status message updated.'));
    }
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $status;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $entity_crud_alter_status_message = $this->entity;

    // Build id.
    $id_array = [
      $form_state->getValue('entity_type'),
      $form_state->getValue('entity_bundle'),
      $form_state->getValue('crud_action'),
    ];
    $id = implode('.', array_filter($id_array)) ?: '_';
    $form_state->setValue('id', $id);

    // If the message alter exists and it's not this one, throw an error.
    if (EntityCrudAlterStatusMessage::load($id) && $entity_crud_alter_status_message->id() !== $id) {
      $url = Url::fromRoute('entity.entity_crud_alter_status_message.edit_form', ['entity_crud_alter_status_message' => $id])->toString();
      $this->messenger()->addError($this->t('The entity CRUD alter status message %id already exists. <a href=":url">Edit it</a>.', [
        '%id' => $id,
        ':url' => $url,
      ]));
      $form_state->setErrorByName('entity_type');
      $form_state->setErrorByName('entity_bundle');
      $form_state->setErrorByName('crud_action');
    }
  }

}
