<?php

namespace Drupal\Tests\entity_crud_alter_status_message\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests overall functionality.
 *
 * @group entity_crud_alter_status_message
 */
class EntityCrudAlterStatusMessageTest extends BrowserTestBase {

  use EntityCrudAlterStatusMessageTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'entity_crud_alter_status_message',
    'node',
    'taxonomy',
    'media',
    'field',
    'field_ui',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The admin user to use during the test.
   *
   * @var \Drupal\user\UserInterface
   */
  private $adminUser;

  // Constants for test values.
  private const TEST_NODE_TITLE = 'Article test node';
  private const TEST_TERM_NAME = 'Test term';
  private const TEST_IMAGE_NAME = 'Test image';
  private const TEST_NODE_UPDATED_TITLE = 'Article test node updated';
  private const TEST_TERM_UPDATED_NAME = 'Test term updated';
  private const TEST_IMAGE_UPDATED_NAME = 'Test image updated';

  /**
   * Perform initial setup tasks that run before every test method.
   */
  public function setUp(): void {
    parent::setUp();

    // Create article content type.
    $this->drupalCreateContentType([
      'type' => 'article',
      'name' => 'Article',
    ]);

    // Add vocabulary "tags".
    $this->addVocabulary(['name' => 'tags', 'vid' => 'tags']);

    // Create image media type.
    $this->drupalCreateMediaType([
      'id' => 'image',
      'label' => 'Image',
      'source' => 'file',
    ]);

    // Create the admin user.
    $this->adminUser = $this->DrupalCreateUser([
      'administer entity_crud_alter_status_message',
      'create article content',
      'create terms in tags',
      'create media',
      'edit any article content',
      'edit terms in tags',
      'edit any image media',
    ]);

    // Create test entities.
    $this->createTestEntities();

  }

  /**
   * Create test entities for CRUD operations.
   */
  private function createTestEntities() {
    $this->createTestEntityCrudAlterStatusMessage('node', 'article', 'default', 'Test create node title:@label edit:@edit_link');
    $this->createTestEntityCrudAlterStatusMessage('taxonomy_term', 'tags', 'default', 'Test create term title:@label edit:@edit_link');
    $this->createTestEntityCrudAlterStatusMessage('media', 'image', 'add', 'Test create image title:@label edit:@edit_link');
    $this->createTestEntityCrudAlterStatusMessage('node', 'article', 'edit', 'Test update node title:@label edit:@edit_link');
    $this->createTestEntityCrudAlterStatusMessage('taxonomy_term', 'tags', 'edit', 'Test update term title:@label edit:@edit_link');
    $this->createTestEntityCrudAlterStatusMessage('media', 'image', 'edit', 'Test update image title:@label edit:@edit_link');
  }

  /**
   * Tests the entity_crud_alter_status_message functionality.
   */
  public function testEntityCrudAlterStatusMessage() {
    $this->drupalLogin($this->adminUser);
    $assert_session = $this->assertSession();

    // Define create tests.
    $create_tests = [
      [
        'form_path' => '/node/add/article',
        'field' => 'title[0][value]',
        'value' => self::TEST_NODE_TITLE,
        'expected_text' => '/Test create node title:' . self::TEST_NODE_TITLE . ' edit:.*\/node\/\d*\/edit/',
      ],
      [
        'form_path' => '/admin/structure/taxonomy/manage/tags/add',
        'field' => 'name[0][value]',
        'value' => self::TEST_TERM_NAME,
        'expected_text' => '/Test create term title:' . self::TEST_TERM_NAME . ' edit:.*\/taxonomy\/term\/\d*\/edit/',
      ],
      [
        'form_path' => '/media/add/image',
        'field' => 'name[0][value]',
        'value' => self::TEST_IMAGE_NAME,
        'expected_text' => '/Test create image title:' . self::TEST_IMAGE_NAME . ' edit:.*\/media\/\d*\/edit/',
      ],
    ];

    // Execute create tests.
    $this->executeTests($create_tests, $assert_session);

    // Define update tests.
    $update_tests = [
      [
        'form_path' => '/node/1/edit',
        'field' => 'title[0][value]',
        'value' => self::TEST_NODE_UPDATED_TITLE,
        'expected_text' => '/Test update node title:' . self::TEST_NODE_UPDATED_TITLE . ' edit:.*\/node\/\d*\/edit/',
      ],
      [
        'form_path' => '/taxonomy/term/1/edit',
        'field' => 'name[0][value]',
        'value' => self::TEST_TERM_UPDATED_NAME,
        'expected_text' => '/Test update term title:' . self::TEST_TERM_UPDATED_NAME . ' edit:.*\/taxonomy\/term\/\d*\/edit/',
      ],
      [
        'form_path' => '/media/1/edit',
        'field' => 'name[0][value]',
        'value' => self::TEST_IMAGE_UPDATED_NAME,
        'expected_text' => '/Test update image title:' . self::TEST_IMAGE_UPDATED_NAME . ' edit:.*\/media\/\d*\/edit/',
      ],
    ];

    // Execute update tests.
    $this->executeTests($update_tests, $assert_session);
  }

  /**
   * Execute the tests for create and update operations.
   */
  private function executeTests(array $tests, $assert_session) {
    foreach ($tests as $test) {
      // Navigate to the form.
      $this->drupalGet($test['form_path']);
      $edit = [
        $test['field'] => $test['value'],
      ];
      // Submit the form.
      $this->submitForm($edit, 'edit-submit');
      // Verify that the entity_crud_alter_status_message record is displayed.
      $assert_session->pageTextMatches($test['expected_text']);
    }
  }

}
